/////////////////////////////////////////////////////////////////////////////////////////
//
// MC9S08QG4 sample for Freescale USBSPYDER08
//
// This program continuously varies the brightness of the LED by using a Pulse Width
// Modulation (PWM) technique.
//
// If you turn an LED on and off fast enough then it will appear to stay on continuously 
// and since there is less current flowing overall, the LED will appear less bright.
// By varying the duty cycle of the signal used to turn the LED on and off, we can vary
// the LED brightness.
// In this example, the LED varies its brightness from a minimum value to a maximum value
// and back to a minum value every second and a half. During this period, the LED is
// driven on and off about 200 times, each time with different on/off durations.
// --------------------------------------------------------------------------------------
//
// This project has been written for CodeWarrior 5.1 for HC(S)08
// and uses Registers Files MC9S08QG4.H and MC9S08QG4.C, version 2.87.095
//         												  
// --------------------------------------------------------------------------------------
// Copyright (c) 2006 SofTec Microsystems
// http://www.softecmicro.com/
//
/////////////////////////////////////////////////////////////////////////////////////////

#include <hidef.h>
// #include <stdlib.h>
#include "derivative.h"

#define TIMER_STOP                0x00
#define TIMER_START               0x4A  // Configures Timer1 with prescaler 4 and fBUS as source cock
    
#define DUTY_CYCLE_MAX            98    // %
#define DUTY_CYCLE_MIN            2     // %       
#define TIMER_OVERFLOW            15000 // us
#define DUTY_CYCLE_INCREMENT      1     // %

#define FREQ_BUS              8000000  // 8 MHz
#define NUM_BUS_CYCLES        (FREQ_BUS * 100)/1000000
#define DELAY_X_VALUE         (((((NUM_BUS_CYCLES - 6)*10)/4)+5)/10)-1
#define DELAY_TIME            500  // ms

// ICSTRM location in Flash
unsigned char ICSTRM_FLASH @0xFFAF;

// Flash security (NVOPT Register)
const unsigned char FLASH_SECURITY  @0xFFBF = 0xFE;

// unsigned int duty_cycle_variation;

unsigned char line;           // Line number to be displayed (placed in RAM at 0x100)
unsigned char txtbuff[20];    // Text buffer for one line of display (placed in RAM at 0x101)
 
/////////////////////////////////////////////////////////////////////////////////////////
// PeriphInit
// --------------------------------------------------------------------------------------
// Initializes various registers and peripherals
/////////////////////////////////////////////////////////////////////////////////////////
void PeriphInit (void)
{  
  // Disables COP
  SOPT1 &= 0x7F;
  
  // Configures PTA0,1 & 3 as output
  PTAD = 0x00;
  PTADD = 0x0F; // (1+2+8)
 
  // Configures ICS clock. Selects FLL Engaged Internal Mode (Divides ICSOUT by 1)
  // fBUS about 8 MHz
  ICSTRM = ICSTRM_FLASH;
  ICSC2 = 0x00;
  ICSC1 = 0x04;
 
  // Waits until output FLL is selected (CLKST=00)
  while (ICSSC & 0x0C)
    ; 
/*       
  // Timer1 period
  TPMMOD = TIMER_OVERFLOW;
  // Initial duty cycle 
  TPMC0V = TIMER_OVERFLOW / 100 * DUTY_CYCLE_MIN;  
  // Selects PWM Low-true pulses in the Timer1 Channel0 (PTA0/TPM1CH0) 
  TPMC0SC = 0x024;
  // Starts Timer1 
  TPMSC = TIMER_START;
*/
}

//=======================================================================================
// MODULE: LCD interface for HD44780 - 4bit communication mode
//=======================================================================================
// Display: EL2004D - 20x4 chars
//
// Connection:
//
// S08QG4           HC164            HD44780
// -----+          +------+         +--------+
//  PTA0|8--------8|CLK Q0|3------11|DB4  DB0|7-------+
//  PTA1|7------+-1|DA  Q1|4------12|DB5  DB1|8-------+
//  PTA2|6      +-2|DB  Q2|5------13|DB6  DB2|9-------+
//  PTA3|5-+       |    Q3|6------14|DB7  DB3|10------+    
// -----+  |  5V<-o|_R  Q4|10------4|RS   GND|1-------+    
//         |      9+------+  +-----6|E    VCC|2--->5V |      
//         +-----------------+  +--5|R/_W CTR|3->0-5V |    
//                              |   +--------+        |    
//                             ^^^                   ^^^
// CTR: Display contrast is connected to middle tip of 10k pot between 5V and GND

/////////////////////////////////////////////////////////////////////////////////////////
// Delay_100us
// --------------------------------------------------------------------------------------
// This subroutine performs a specified delay in multiples of 100 us
/////////////////////////////////////////////////////////////////////////////////////////
void Delay_100us (unsigned char hundreds_us)
{
  asm
  {
    lda hundreds_us
    beq exit
    delay_us:
      ldx #DELAY_X_VALUE          ; [2]
    loop:
      dbnzx loop                  ; [4]
    dbnza delay_us                ; [4]
    exit:
  }
}

/////////////////////////////////////////////////////////////////////////////////////////
// Delay_100us
// --------------------------------------------------------------------------------------
// This subroutine performs fixed delay of half of a second
/////////////////////////////////////////////////////////////////////////////////////////
void Wait500ms(void) {
int i;

// Waits 500 ms
for (i=0; i<DELAY_TIME; i++)
      Delay_100us(10);  // 1 ms delay

}

/////////////////////////////////////////////////////////////////////////////////////////
// ToggleCLK
// --------------------------------------------------------------------------------------
// Switch HC164 clock ON for 1ms and OFF again for another 1ms
/////////////////////////////////////////////////////////////////////////////////////////
void ToggleCLK(void) {
//unsigned char n;
//  for (n = 0; n < 2; n++)   
//  {
    // Toggles CLK
    PTAD ^= 0x01;
    Delay_100us(10);  // 1 ms delay
    PTAD ^= 0x01;
    Delay_100us(10);  // 1 ms delay
    
//   }
}

/////////////////////////////////////////////////////////////////////////////////////////
// BlipE
// --------------------------------------------------------------------------------------
// Blip E signal on LCD display (1ms ON, OFF)
/////////////////////////////////////////////////////////////////////////////////////////
void BlipE(void) {
  PTAD |= 0x08;     // lcdE = 1
  Delay_100us(1);		// 100us delay
  PTAD &= ~0x08;    // lcdE = 0
  Delay_100us(1);   // 100us delay
}


/////////////////////////////////////////////////////////////////////////////////////////
// LCDNibble
// --------------------------------------------------------------------------------------
// Output 4-bit data + R/S bit to shift register
/////////////////////////////////////////////////////////////////////////////////////////
void LCDNibble (unsigned char w) {
unsigned char i, d;  

/*
for (de = 0; de < 1; de++) {
    Delay_100us(1);
}
*/
for (i = 0; i < 5; i++) { // Repeat for 4-bits

    d = (unsigned char) w & (unsigned char) 0x10;       // Check bit4 of Working reg. (bit order RS, b3, b2, b1, b0)
    switch (d) {          // And based on its state
      case (0x10): 
        PTAD |= 0x02;     // Set D pin on ShReg to log. 1
        break;
      case (0):
        PTAD &= ~0x02;    // Set D pin on ShReg to log. 0
        break;
    } // End switch
    
    ToggleCLK();          // Shift out data with clock toggle
    
    w <<= 1;              // Proceed with next bit
    
  } // End for
    
} // End routine

/////////////////////////////////////////////////////////////////////////////////////////
// LCDOut
// --------------------------------------------------------------------------------------
// Output one character to LCD display connected via Shift Register (HC164) 
/////////////////////////////////////////////////////////////////////////////////////////
void LCDOut (unsigned char chDat) 
{
unsigned char w;
  
  w = ((chDat) >> 4 );
  w |= 0x10;              // Set RS to log.1
  LCDNibble(w);
  BlipE();                // Commit the information to display
  
  w = ((chDat) & 0x0F);
  w |= 0x10;              // Set RS to log.1
  LCDNibble(w);
  BlipE();                // Commit the information to display
  
}

 /* Print the next character and increment the pointer "nextchr" and        */
  /*      decrement the counter "howmany"                                    */
/*
  void LCDPrintNext(void) {

    if (howmany-- > 0) {
      lcdRS = 1;
      #ifdef lcd4bit
        lcdPort &= PORTMASK;
        lcdPort |= OUTNUMBER(((*nextchr)>>0x04));
        LCDSend();
        lcdPort &= PORTMASK;
        lcdPort |= OUTNUMBER(((*nextchr) & 0x0F));
        LCDSend();
      #else
        lcdPort = (*nextchr);
        LCDSend();
      #endif
      nextchr++;
      timerLCD = gTimeBaseInterruptperms;
      LCDInternalStatus = lcdStatusPrinting;
    }
    else {
      LCDInternalStatus = lcdStatusReady;
    }

  }
 */

/////////////////////////////////////////////////////////////////////////////////////////
// InitLCD
// --------------------------------------------------------------------------------------
// Intializes LCD to 4-bit operation following the datasheet instructions
/////////////////////////////////////////////////////////////////////////////////////////
void InitLCD() {

unsigned char i;

// Pre-init:
// 1. Reset shift reg (8x log.0)
// 2. Set R/S to log. 0

PTAD = 0x00;      // Clear everything on the port
 
// Reset shift reg 
for (i=0; i<8; i++) {
  ToggleCLK();
}

// SEE: Page 46 of HD 44780 manual
// Wait 15ms - 40 ms
Delay_100us(200);       // 20ms delay 
Delay_100us(200);       // 20ms delay 

// Data: #00000011b
LCDNibble(0x03);

// Wait > 4.1 ms
Delay_100us(50);				// 5ms delay

// BlipE
BlipE();
// Wait > 100us
Delay_100us(2);         // 200us delay

// 2x Blip E
BlipE();
BlipE();

// Data: #00000010b
LCDNibble(0x02);

// 2x BlipE
BlipE();
BlipE();

// Data: #00001000b
LCDNibble(0x08);          // Number of lines = 2, Font = 5*8 (N=1,F=0,*,*)
// BlipE
BlipE();


// 1st init byte
LCDNibble(0x00);
BlipE();

LCDNibble(0x0E);
BlipE();

// 2nd init byte
LCDNibble(0x00);
BlipE();

LCDNibble(0x06);
BlipE();

// 3rd init byte
LCDNibble(0x00);
BlipE();

LCDNibble(0x01);
BlipE();


// DISPLAY IS NOW READY TO RECEIVE AND PRINT CHARACTERS
}

/////////////////////////////////////////////////////////////////////////////////////////
// LCDClear
// --------------------------------------------------------------------------------------
// Clear the content of LC display
/////////////////////////////////////////////////////////////////////////////////////////
void LCDClear(void) {
  LCDNibble(0x00);
  BlipE();
  LCDNibble(0x01);
  BlipE();
}



void LCDCursor(unsigned char dd) {

unsigned char ddr;

  ddr = (dd | 0x80) >> 4;
  LCDNibble(ddr);
  BlipE();
  
  ddr = dd & 0x0F;
  LCDNibble(ddr);
  BlipE();
}


/////////////////////////////////////////////////////////////////////////////////////////
// LCDLineSel
// --------------------------------------------------------------------------------------
// Set the line on the display for further writing
/////////////////////////////////////////////////////////////////////////////////////////
void LCDLineSel(int sline) {

    switch (sline) {
    case (0):
        LCDCursor(0x00);
        break;
    case (1):
        LCDCursor(0x40);
        break;
    case (2):
        LCDCursor(0x14);
        break;
    case (3):
        LCDCursor(0x54);
        break;
    default:
        break;
    }
}

/////////////////////////////////////////////////////////////////////////////////////////
// LCDRefresh
// --------------------------------------------------------------------------------------
// Read the content of buffer and print it on display
/////////////////////////////////////////////////////////////////////////////////////////

void LCDRefresh(void) {

unsigned char j;
unsigned char w;

// Refresh 4 lines of display (not enough room for all lines)
// for (i=0; i < 4; i++) {
  
  // i = line;
  if (line < 4) {
  
    LCDLineSel(line);
    
    for (j=0; j < 20; j++) {          // 20 chars per line
      w = txtbuff[j];
      LCDOut(w);
    } // End For
    
  } // End If 

} // End Refresh

/////////////////////////////////////////////////////////////////////////////////////////
// InitBuff
// --------------------------------------------------------------------------------------
// Intializes buffer for text storage
/////////////////////////////////////////////////////////////////////////////////////////
void InitBuff(void) {
unsigned char i;

  for (i=0; i < 20; i++) {
    txtbuff[i] = ' ';
  }
  
  line = 3;                         // Last line will be cleared
}


/////////////////////////////////////////////////////////////////////////////////////////
// MAIN
// --------------------------------------------------------------------------------------
// Entry point
/////////////////////////////////////////////////////////////////////////////////////////
void main (void) 
{
  unsigned char i,j;
  
  EnableInterrupts;
  PeriphInit();

  InitBuff();  
    
  InitLCD();
  LCDClear();
//  Wait500ms();


// Display test
// Shows line numbers on the display for 500ms
  for(j=0;j<4;j++) {
    
    LCDLineSel(j);
    for(i=0;i<20;i++) {
      LCDOut('1'+j); 
    }
  }
  Wait500ms();

// Main loop -> display refresh every 20ms
for (;;) {
  
  LCDRefresh();
  Delay_100us(200);     // 20ms delay
}
  /* */
}


/////////////////////////////////////////////////////////////////////////////////////////
// Timer1Overflow
// --------------------------------------------------------------------------------------
// TIMER1 Overflow Interrupt 
/////////////////////////////////////////////////////////////////////////////////////////
interrupt void Timer1Overflow (void) 
{
/*
  // Stops Timer1
  TPMSC = TIMER_STOP;
  
  // Checks for duty cycle limits
  if (TPMC0V >= TIMER_OVERFLOW / 100 * DUTY_CYCLE_MAX)      // Maximum duty cycle reached
    duty_cycle_variation = -DUTY_CYCLE_INCREMENT;
  else if (TPMC0V <= TIMER_OVERFLOW / 100 * DUTY_CYCLE_MIN) // Minimum duty cycle reached
    duty_cycle_variation = DUTY_CYCLE_INCREMENT;  
  
  // Updates
  TPMC0V += TIMER_OVERFLOW / 100 * duty_cycle_variation;

  // Restarts Timer1
  TPMSC = TIMER_START;  
*/
}
   


/*
// Predko's HC174 LCD interface pseudo routine
LCDNybble(char Nybble, char RS)
{

int i;


  Data = 0;                      //  Clear the '174
  for (i = 0; i < 6; i++) {      //  Repeat for six bits
    Clock = 1; Clock = 0;        //  Write the "0"s into the '174
  }

  Data = 1;                      //  Output the "AND" Value
  Clock = 1; Clock = 0;

  Data = RS;                     //  Output the RS Bit Value
  Clock = 1; Clock = 0;

  for (i = 0; i < 4; i++) {      // Output the Nybble
    if ((Nybble & 0x008) != 0)
      Data = 1;                  // Output the High Order Bit
    else
      Data = 0;
    Clock = 1; Clock = 0;        //  Strobe the Clock
    Nybble = Nybble << 1;        // Shift up Nybble for Next Byte
  }

  Clock = 1; Clock = 0;          // Toggle the "E" Clock Bit

} // End LCDNybble
*/
